package com.mall.client.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mall.client.mapper.SkuMapper;
import com.mall.client.service.*;
import com.mall.common.constant.TokenConstants;
import com.mall.common.domain.*;
import com.mall.common.domain.request.SkuRequest;
import com.mall.common.domain.vo.SkuVo;
import com.mall.common.redis.RedisCache;
import com.mall.common.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

import static com.fasterxml.jackson.databind.util.ClassUtil.name;

/**
 * sku业务实现类
 * @Author: lzh
 * @CreateTime: 2024-04-25  11:44
 * @Description: TODO
 * @Version: 1.0
 */
@Service
public class SkuServiceImpl extends ServiceImpl<SkuMapper, Sku> implements SkuService {
    @Autowired
    private SkuMapper skuMapper;

    @Autowired
    private SpikesService spikesService;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private BrandService brandService;

    @Autowired
    private SpuService spuService;

    @Autowired
    private RedisCache redisCache;

    @Override
    public Result selectSku(SkuVo skuVo) {
        PageHelper.startPage(skuVo.getPageNum(),skuVo.getPageSize());
        List<Sku> skuList = this.list();
        PageInfo<Sku> skuPageInfo = new PageInfo<>(skuList);

        skuPageInfo.getList().forEach(sku -> {
            LambdaQueryWrapper<Brand> brandLambdaQueryWrapper = new LambdaQueryWrapper<>();
            brandLambdaQueryWrapper.eq(Brand::getId,sku.getBrandId());
            List<Brand> brandList = this.brandService.list(brandLambdaQueryWrapper);
            skuVo.setBrandList(brandList);

            LambdaQueryWrapper<Spu> spuLambdaQueryWrapper = new LambdaQueryWrapper<>();
            spuLambdaQueryWrapper.eq(Spu::getId,sku.getSpuId());
            List<Spu> spuList = this.spuService.list(spuLambdaQueryWrapper);
            skuVo.setSpuList(spuList);
        });
        skuVo.setSkuList(skuPageInfo.getList());
        return Result.success(skuVo);
    }

    @Override
    public Result<UserInfo> info() {
        String token = request.getHeader(TokenConstants.LOGIN_TOKEN_KEY);
        UserInfo userInfo = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + token);
        return Result.success(userInfo);
    }

    @Override
    public Result insertSku(SkuRequest skuRequest) {
        Spu spu = Spu.builder()
                .categoryId(skuRequest.getCategoryId())
                .brandId(skuRequest.getBrandId())
                .name(skuRequest.getName())
                .createTime(new Date())
                .updateTime(new Date())
                .build();
        spuService.save(spu);

        Long spuId = spu.getId();
        Sku sku = Sku.builder()
                .spuId(spuId)
                .name(skuRequest.getName())
                .categoryId(skuRequest.getCategoryId())
                .brandId(skuRequest.getBrandId())
                .defaultImage(skuRequest.getDefaultImage())
                .title(skuRequest.getTitle())
                .subtitle(skuRequest.getSubtitle())
                .price(skuRequest.getPrice())
                .weight(skuRequest.getWeight())
                .inventory(skuRequest.getInventory())
                .build();
        this.save(sku);
        return Result.success(true,"添加商品成功");
    }

    @Override
    public Result infoSku(Long id) {
        Sku sku = this.getById(id);
        this.infoSpu(sku.getSpuId());
        return Result.success(sku,"商品Sku回显");
    }

    public Result infoSpu(Long spuId){
        Spu spu = spuService.getById(spuId);
        return Result.success(spu,"商品Spu回显");
    }

    @Override
    public Result updateSku(SkuRequest skuRequest) {
        Sku sku = Sku.builder()
                .id(skuRequest.getSkuId())
                .name(skuRequest.getName())
                .categoryId(skuRequest.getCategoryId())
                .brandId(skuRequest.getBrandId())
                .defaultImage(skuRequest.getDefaultImage())
                .title(skuRequest.getTitle())
                .subtitle(skuRequest.getSubtitle())
                .price(skuRequest.getPrice())
                .activityPrice(skuRequest.getPrice())
                .weight(skuRequest.getWeight())
                .inventory(skuRequest.getInventory())
                .build();
        this.updateById(sku);

        Spu spu = Spu.builder()
                .categoryId(skuRequest.getCategoryId())
                .brandId(skuRequest.getBrandId())
                .name(skuRequest.getName())
                .updateTime(new Date())
                .build();
        this.spuService.updateById(spu);

        if (skuRequest.getSpikesId() != null) {
            Spikes spikes = Spikes.builder()
                    .spikesId(skuRequest.getSpikesId())
                    .spikesTitle(skuRequest.getTitle())
                    .spikesUnit(skuRequest.getSubtitle())
                    .spikesManyImg(skuRequest.getDefaultImage())
                    .spikesImg(skuRequest.getDefaultImage())
                    .spikesThings(skuRequest.getSubtitle())
                    .build();
            this.spikesService.updateById(spikes);
        }
        return Result.success(true,"修改成功");
    }
}