package com.mall.client.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mall.client.mapper.SpikesMapper;
import com.mall.client.service.SkuService;
import com.mall.client.service.SpikesService;
import com.mall.common.domain.Sku;
import com.mall.common.domain.Spikes;
import com.mall.common.domain.vo.SeckillVo;
import com.mall.common.redis.RedisCache;
import com.mall.common.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.stream.Collectors;

/**
 * 秒杀场次业务实现类
 * @Author: lzh
 * @CreateTime: 2024-04-25  11:05
 * @Description: TODO
 * @Version: 1.0
 */
@Service
public class SpikesServiceImpl extends ServiceImpl<SpikesMapper, Spikes> implements SpikesService {
    @Autowired
    private SpikesMapper spikesMapper;

    @Autowired
    private SkuService skuService;

    @Autowired
    private RedisCache redisCache;

    @Override
    public Result selectSpikes(SeckillVo seckillVo) {
        if (redisCache.hasKey(seckillVo.getSpikesId().toString())) {
            return Result.error("秒杀活动已结束");
        }
        //从redis中获取秒杀活动信息
        Spikes spikes = redisCache.getCacheObject(seckillVo.getSpikesId().toString());
        return Result.success(spikes);
    }

    @Override
    public Result insertSpikes(SeckillVo seckillVo) {
        if (redisCache.hasKey(seckillVo.getSpikesId().toString())) {
            return Result.error("秒杀活动已存在");
        }

        Spikes spikes = Spikes.builder()
                .spikesTitle(seckillVo.getSpikesTitle())
                .spikesUnit(seckillVo.getSpikesUnit())
                .spikesNum(seckillVo.getSpikesNum())
                .spikesState(seckillVo.getSpikesState())
                .spikesImg(seckillVo.getSpikesImg())
                .spikesManyImg(seckillVo.getSpikesManyImg())
                .spikesThings(seckillVo.getSpikesThings())
                .endTime(seckillVo.getEndTime())
                .build();
        this.save(spikes);

        skuService.updateById(
                (Sku) seckillVo.getSkuList().stream().map(sku ->
                        Sku.builder()
                            .id(sku.getSkuId())
                            .spikesId(spikes.getSpikesId())
                            .activityPrice(sku.getActivityPrice())
                            .inventoryRestrict(sku.getInventoryRestrict())
                            .build()
                ).collect(Collectors.toList())
        );
        return Result.success(200,"添加秒杀信息成功");
    }
}