package com.mall.client.sync;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.mall.client.service.SkuService;
import com.mall.client.service.SpikesService;
import com.mall.common.domain.Sku;
import com.mall.common.domain.Spikes;
import com.mall.common.domain.vo.SeckillVo;
import com.mall.common.redis.RedisCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Component
public class TimingDepositSeckill {
    @Autowired
    private SkuService skuService;
    @Autowired
    private SpikesService spikesService;
    @Autowired
    private RedisCache redisCache;

    //每天的0点执行,将秒杀的数据存入redis中
    @Scheduled(cron = "0 0 0 * * ?")
    public void timingDepositSeckill() {
        ArrayList<SeckillVo> seckillList = new ArrayList<>();

        List<Spikes> list = spikesService.list();

        //判断秒杀活动时间是否 大于或等于 当前时间 是 就放入redis
        for (Spikes spikes : list) {
            SeckillVo seckillVo = new SeckillVo();
            //判断活动时间是否 大于或等于 当前时间 是 就放入redis
            if (spikes.getCreateTime().getTime() >= System.currentTimeMillis()) {

                LambdaQueryWrapper<Sku> skuLambdaQueryWrapper = new LambdaQueryWrapper<>();

                //查询条件根据sku中的spikesId
                skuLambdaQueryWrapper.eq(Sku::getSpikesId,seckillVo.getSpikesId());

                //查询sku开启秒杀的信息
                List<Sku> skuList = skuService.list(skuLambdaQueryWrapper);

                //秒杀的商品信息存入
                seckillVo.setSkuList(skuList);

                //秒杀的基本信息
                seckillVo.setSpikesId(spikes.getSpikesId());
                seckillVo.setSpikesImg(spikes.getSpikesImg());
                seckillVo.setSpikesManyImg(spikes.getSpikesManyImg());
                seckillVo.setSpikesThings(spikes.getSpikesThings());
                seckillVo.setSpikesTitle(spikes.getSpikesTitle());
                seckillVo.setCreateTime(spikes.getCreateTime());
                seckillVo.setEndTime(spikes.getEndTime());
            }
            seckillList.add(seckillVo);
        }
        for (SeckillVo seckillVo : seckillList) {
            //判断信息是否大于当天时间 如果大于不予处理
            if (seckillVo.getCreateTime().getTime() > System.currentTimeMillis() + 24 * 60 * 60 * 1000) {
                continue;
            }
            //将秒杀的数据 存入redis中 并设置过期时间 为 秒杀活动 结束的时间 减去 凌晨
            redisCache.setCacheObject("seckill:"+seckillVo.getSpikesId(),seckillVo,seckillVo.getEndTime().getTime()-System.currentTimeMillis(), TimeUnit.MILLISECONDS);
            //如果redis中信息已经删除 将Spikes中的isDelete 修改为1 （在是今天的活动的前提下）
//            if (redisCache.getCacheObject("seckill:"+seckillVo.getSpikesId()) == null) {
//                spikesService.update().eq("spikes_id",seckillVo.getSpikesId()).set("is_delete",1).update();
//            }
        }
    }
}